'use strict';

var util = require('util');
var WebSocket = require('ws');

var Log = require("../utils/log");
var constant = require("../utils/constant");
var DBusProxy = require("./dbus-proxy/DBusProxy");

function WSServer(port) {
    var self = this;
    this.wss = new WebSocket.Server({"port": port, "host" : "127.0.0.1"}); /*{

    host {String}：服务绑定的主机名

    port {Number}：服务绑定的端口

*/
    this.wss.on('connection', function (ws, req) {
        ws.on('message', function (message) {
            // Log.i(TAG, "handleIncomingMessage = " + message);
            try {
                var obj = JSON.parse(message);
                self.handleIncomingMessage(ws, obj);
            } catch (e) {
                console.log('WSServer on message failed:', e.message);
                console.log(e.stack);
            }
        });
        ws.on('close', function () {
            self.handleClose(ws);
        });

        ws.on('error', function (err) {
            self.handleError(ws, err);
        });
    });
}

WSServer.prototype.serverSendMessageToFC = function (info) {
    // Log.i("WSServer", "serverSendMessageToFC begin args = ", JSON.stringify(info));
    var cnt = 0;
    if(this.wss.clients) {
        this.wss.clients.forEach(function each(client) {
            if(client.readyState === WebSocket.OPEN) {
                client.send(JSON.stringify(info));
            }
            cnt++;
        });
    }
    Log.i("WSServer", "connect client size = " + cnt);
};

WSServer.prototype.setDbusProxy = function (proxy) {
    this.dbusproxy = proxy;
}

WSServer.prototype.setIFTTT = function (ifttt) {
    this.ifttt = ifttt;
}

WSServer.prototype.setStateMachine = function (statemachine) {
    this.stateMachine = statemachine;
}

WSServer.prototype.setRequestHandler = function (requestHandler) {
  this.requestHandler = requestHandler;
};

function res(tag, err, data, ws) {
  var msg = wss.wrapResponse(tag, err, data);
  if (ws.readyState === WebSocket.OPEN) {
    ws.send(JSON.stringify(msg));
  }
}

WSServer.prototype.handleIncomingMessage = function (ws, req) {
    var cmd = req['cmd'];
    var tag = req['tag'];
    if (!cmd || !tag) {
        Log.i("WSServer", `invalid incoming message: ${JSON.stringify(req)}, drop!`);
        return;
    }
    var id = req['id'];
    if (cmd === 'setProperty') {
        var values = req['values'];
        this.setProperty(tag, id, values, ws);
    } else if (cmd === 'getProperty') {
        var keys = req['keys'];
        this.getProperty(tag, id, keys, ws);
    } else if (cmd === 'callService') {
        var service = req['service'];
        var params = req['params'];
        this.callService(tag, id, service, params, ws);
    } else if (cmd === 'sendJobExecuteStatus') {
        var status = req['status'];
        this.sendJobExecuteStatus(tag, id, status, ws);
    } else if (cmd === 'getDeviceList') {
        var status = req['status'] || "";
        this.getDeviceList(tag, status, ws);
    } else if (cmd === 'getDeviceProfile') {
        this.getDeviceProfile(tag, id, ws);
    } else if(cmd === 'publish') {
        var wellknownName = req["wellknownName"];
        var _interface = req["_interface"];
        var objectPath = req["objectPath"];
        var messageName = req["messageName"];
        var payload = req["payload"];
        this.publish(tag, wellknownName, _interface, objectPath, messageName, payload, ws);
    } else if(cmd === 'notifyIotMessage') {
        var message = req["message"];
        this.notifyIotMessage(tag, message, ws);
    } else if(cmd === 'operateScene') {
        var id = req["id"];
        var operation = req["operation"];
        this.operateScene(tag, id, operation, ws);
    } else if (cmd === 'request') {
        if (this.requestHandler) {
            this.requestHandler.handleRequest(req, ws);
        } else {
          res(tag, new Error('Request handler has not been set!'), null, ws);
        }
    } else {
        res(tag, new Error(`Unknown operation "${cmd}".`), null, ws);
    }
};

WSServer.prototype.handleClose = function (ws) {

};

WSServer.prototype.handleError = function (ws, error) {

};

WSServer.prototype.wrapResponse = function (tag, err, ret) {
    var res = {};
    res['tag'] = tag;
    if (err) {
        res['status'] = 'failed';
        if(Object.prototype.toString.call(err) === '[object Object]') {
            res['error'] = JSON.stringify(err);
        } else if(Object.prototype.toString.call(err) === '[object Error]') {
            res['error'] = err.toString();
        }
    } else {
        res['status'] = 'success';
        res['value'] = ret;
    }
    return res;
};

WSServer.prototype.setProperty = function (tag, id, values, ws) {
    var self = this;
    this.dbusproxy.setBatchPropertiesAsync(id, values, function (err, ret) {
        var res = self.wrapResponse(tag, err, ret);
        if (ws.readyState === WebSocket.OPEN) {
            ws.send(JSON.stringify(res));
        }
    });
};

WSServer.prototype.getProperty = function (tag, id, values, ws) {
    var self = this;
    this.dbusproxy.getBatchPropertiesAsync(id, values, function (err, ret) {
        var res = self.wrapResponse(tag, err, ret);
        if (ws.readyState === WebSocket.OPEN) {
            ws.send(JSON.stringify(res));
        }
    });
};

WSServer.prototype.callService = function (tag, id, service, params, ws) {
    var self = this;
    this.dbusproxy.callServiceAsync(id, service, params, function (err, ret) {
        var res = self.wrapResponse(tag, err, ret);
        if (ws.readyState === WebSocket.OPEN) {
            ws.send(JSON.stringify(res));
        }
    });
};
// 发送job的执行状态，需回复
WSServer.prototype.sendJobExecuteStatus = function (tag, jobId, status, ws) {
  var self = this;
  SceneResultSender.get().send(status, function (err) {
    var res = self.wrapResponse(tag, null, true);
    if (ws.readyState === WebSocket.OPEN) {
      Log.i("WSServer", "sendJobExecuteStatus res", JSON.stringify(res));
      ws.send(JSON.stringify(res));
    }
  });
};

WSServer.prototype.publish = function (tag, wellknowName, _interface, objectPath, messageName, payload, ws) {
    Log.i("WSServer", "publish " + messageName + " payload = " + payload);
    DBusProxy.proxy.sendTargetSignal(wellknowName, objectPath, _interface, messageName, payload);
    var res = this.wrapResponse(tag, null, true);
    Log.i("WSServer", "publish readyState", ws.readyState);
    if (ws.readyState === WebSocket.OPEN) {
        Log.i("WSServer", "publish res", JSON.stringify(res));
        ws.send(JSON.stringify(res));
    }
};

WSServer.prototype.notifyIotMessage = function (tag, message, ws) {
    Log.i("WSServer", "notifyIotMessage " + JSON.stringify(message));
    var self = this;
    IotMessageSender.get().send(message, function (err) {
        var res = self.wrapResponse(tag, err);
        Log.i("WSServer", "notifyIotMessage readyState", ws.readyState);
        if (ws.readyState === WebSocket.OPEN) {
            Log.i("WSServer", "notifyIotMessage res", JSON.stringify(res));
            ws.send(JSON.stringify(res));
        }
    });
};

WSServer.prototype.operateScene = function (tag, sceneId, operation, ws) {
    Log.i("WSServer", "operateScene sceneId " + sceneId + " , operation " + operation);
    var self = this;
    function callback(err) {
        var res = self.wrapResponse(tag, err);
        Log.i("WSServer", "operateScene readyState", ws.readyState);
        if (ws.readyState === WebSocket.OPEN) {
            Log.i("WSServer", "operateScene res", JSON.stringify(res));
            ws.send(JSON.stringify(res));
        }
    }
    if (operation === 'enable') {
        this.ifttt.enableScene(sceneId, callback);
    } else {
        this.ifttt.disableScene(sceneId, callback);
    }
};

WSServer.prototype.getDeviceList = function (tag, status, ws) {
    var self = this;
    this.dbusproxy.getDeviceListAsync(status, function (err, ret) {
        var res = self.wrapResponse(tag, err, ret);
        if (ws.readyState === WebSocket.OPEN) {
            ws.send(JSON.stringify(res));
        }
    });
};

WSServer.prototype.getDeviceProfile = function (tag, cloudid, ws) {
    var self = this;
    this.dbusproxy.getDeviceProfileAsync(cloudid, function (err, ret) {
        var res = self.wrapResponse(tag, err, ret);
        if (ws.readyState === WebSocket.OPEN) {
            ws.send(JSON.stringify(res));
        }
    });
};

/// IoT Message Sender Begin ///
function IotMessageSender() {
    this.proxy = require('./dbus-proxy/CloudProxy').default();
    var productKey = DBusProxy.proxy.getConfig(constant.GW_PK_KEY);
    var deviceName = DBusProxy.proxy.getConfig(constant.GW_DN_KEY);
    this.topic = util.format('/sys/%s/%s/edge/notification/iotmessage', productKey, deviceName);
}

IotMessageSender.prototype.send = function (message, callback) {
    Log.i('IotMessageSender: topic ' + this.topic + ', message ' + JSON.stringify(message));
    this.proxy.publish(this.topic, message, 0, callback);
};

var iotMessageSender;
IotMessageSender.get = function () {
    if (!iotMessageSender) {
        iotMessageSender = new IotMessageSender();
    }
    return iotMessageSender;
};
/// IoT Message Sender End ///

/// Scene Result Sender Begin ///
function SceneResultSender() {
  this.proxy = require('./dbus-proxy/CloudProxy').default();
  var productKey = DBusProxy.proxy.getConfig(constant.GW_PK_KEY);
  var deviceName = DBusProxy.proxy.getConfig(constant.GW_DN_KEY);
  this.topic = util.format('/sys/%s/%s/edge/rule/post_log', productKey, deviceName);
}

SceneResultSender.prototype.send = function (result, callback) {
  var payload = JSON.stringify(result);
  Log.i('SceneResultSender: topic=%s, payload=%s.', this.topic, payload);
  this.proxy.publish(this.topic, payload, 1, callback);
};

var sceneResultSender;
SceneResultSender.get = function () {
  if (!sceneResultSender) {
    sceneResultSender = new SceneResultSender();
  }
  return sceneResultSender;
};
/// Scene Result Sender End ///

var wss = new WSServer(8099);
var proxy = require("./dbus-proxy/index.js");
wss.setDbusProxy(proxy);
module.exports = wss;
