'use strict';
const util = require("util");
var DBusServer = require("./DBusServer.js");
const CONSTS = require("./ProxyConsts.js");
const config = require('./config');
const Constant = require("../../utils/constant");
const Log = require("../../utils/log");
var broker = require("../../broker/index");
var services = require("./SupportedServices");
const DBusProxy = require("./DBusProxy");
var gw_pk = DBusProxy.proxy.getConfig(Constant.GW_PK_KEY);
var gw_dn = DBusProxy.proxy.getConfig(Constant.GW_DN_KEY);

var EdgeSideServer = function () {
    // this.server = new DBusServer("com.ali.iot.edge.liangqin", "/com/ali/iot/edge/liangqin", "com.ali.iot.edge.liangqin");
    this.server = new DBusServer("iot.runtime.js", "/iot/runtime/js", "iot.runtime.js");

    if (!this.server.prepare()) {
//        console.log("prepare EdgesideServer failed. quit.");
        process.exit(1);
    }
    // JobManager signals
    this.server.addMatch("type='signal',path='/iot/job/manager',interface='iot.job.manager'");
    // DMP signals
    this.server.addMatch("type='signal',path='/iot/dmp/dimu',interface='iot.dmp.dimu'");
    if (config.RUN_MODE === config.MODE_OPTION.MOCK_PROXY_REAL_SERVER_WITH_ALL_DMSG) {
        // for test receive all signal
        this.server.addMatch("type='signal'");
   }
    this.server.addMatch("type='method_call',path='/iot/runtime/js',interface='iot.runtime.js'");

    /**
     * 接收发送到EdgeSide的signal，每个发送过来的消息参数args格式如下：
     * ［id, value]
     * @param  {[type]} signal [description]
     * @param  {[type]} args)  {               } [description]
     * @return {[type]}        [description]
     */
    var self = this;
    this.server.on("signal", onSignal);

    this.server.on("method_call", onMethodCall);
    this.server.on("method_call_with_reply", onMethodCallWithReply);
    this.server.start();
};

  function onSignal(signal, sender, args) {
      Log.i('onSignal={signal: %s, sender: %s, args: %s}', signal, sender, args);
      if (sender === "/org/freedesktop/DBus") return;
      if(sender.indexOf(gw_pk + "/" + gw_dn) !== -1 ) return;

    /*
     * CMP发送的propertiesChanged和设备Events发生，预处理为以下形式
     * (按照约定，CMP只发properties changed和设备的events，不发任何其它signal)：
     * 1.　属性变化：　propertiesChanged
     *     原始signal格式：
     *        　{
     *        　    key: value
     *        　}
     *
     *     参数格式：　
     *             {
     *                 "id": "cloudid",
     *                 "params": {
     *                         "key1": "value1",
     *                         "key2": "value2",
     *                         ...
     *                     }
     *             }
     *
     * 2. Events事件：  ThingEventHappened
     *    原始格式：
     *    sinal即事件名：
     *    ｛
     *        params: {
     *            key: value
     *        }
     *    参数格式：
     *            {
     *                "id": "cloudid",
     *                "eventName": "real_event_name",
     *                "params": {
     *                        // event的参数，和profile中描述一致
     *                    }
     *            }
     *
     */
    var msg = null;
    try {
      if (signal.startsWith("FC")) {
        // We don't care about FC event.
        return;
      } else if (sender.startsWith(CONSTS.CmpPrefix)) {
        var payload = JSON.parse(args);
        if (Object.prototype.toString.call(payload) !== '[object Object]') {
          Log.w('Illegal arguments: %s.', args);
          return;
        }
        var mystr = sender.substr(CONSTS.CmpPrefix.length);
        var pk = mystr.split("/")[0];
        var dn = (mystr.split("/")[1]).split("/")[0];
        var id = pk + "_" + dn;
        if (CONSTS.PropertiesChanged === signal) {
          var params = payload;
          msg = { 'id': id, 'params': params };
        } else {
          var params = payload.params;
          msg = { 'id': id, 'params': params };
          msg.eventName = signal;
          signal = CONSTS.ThingEventHappened;
        }
      } else if ('IFTTTEngine_Event_Invoke' === signal) {
        signal = 'invokeScene';
        msg = JSON.parse(args);
      } else if ('deviceStateChanged' === signal) {
        // We don't care about device state changed event.
        return;
      } else {
        msg = JSON.parse(args);
      }
    } catch (e) {
      Log.w('onSignal failed: %s: %s', signal, e.stack);
      return;
    }
    broker.publish(signal, msg);
  }

function onMethodCall(method, args) {
  Log.i('onMethodCall={method: %s, sender: %s}', method, args);
}

function onMethodCallWithReply(method, sender, args) {
    Log.i('onMethodCallWithReply={method: %s, sender: %s, args: %s}', method, sender, args);
    var values = args.substring(2, args.length -2); //["xxx"]
    if(method === "resetJobs") {
        return services.resetScenes();
    } else if(method === "getSelfDescribing") {
        return services.description();
    } else if(method === "deployJob") {
        return services.deployScenes(values);
    } else if(method === "deleteJob") {
        return services.deleteScenes(values);
    }
}


EdgeSideServer.prototype.registerSignalHandler = function (handler) {
    this.signalHandler = handler;
};

EdgeSideServer.prototype.sendSignal = function (signal, args) {
    return this.server.sendSignal(signal, args);
};

EdgeSideServer.prototype.traceLog = function (level, tag, file, func, line, log) {
    this.server.traceLog(level, tag, file, func, line, log);
};

module.exports = new EdgeSideServer();
