/*
 * Copyright (c) 2019 Alibaba Group Holding Ltd. All rights reversed.
 */

var argv = require('yargs').argv;
var fs = require('fs');
var util = require('util');
var proxy = require('./ws');
var constant = require("../utils/constant");
var Action = require('./actions.js');
var Log = require('../utils/log.js');
var getUUID = require("./getUUID");

var TAG = '[Runner]';

process.on('uncaughtException', function (err) {
  Log.w(TAG, 'Uncaught exception: ', err.stack);
  // Exit after 3s so that logs can be saved.
  setTimeout(function () {
    process.exit(-1);
  }, 3000);
});

function Runner(jobInfo, proxy) {
  this.jobInfo = jobInfo;
  this.config = jobInfo.config;
  this.proxy = proxy;
  this.traceId = getUUID();
  this.sceneid = jobInfo.resourceId;
}

Runner.prototype.isValid = function (sceneObj) {
  return sceneObj && sceneObj[constant.ACTION] && sceneObj[constant.ACTION].length > 0;
};

Runner.prototype.sendJobStatus = function (err, startTime, details, callback) {
  var useTime = new Date() - startTime;
  var summary = this.generateLog(useTime, !err);
  var result = summary + details;
  Log.m(TAG + ' %s(%s) execute result: %s', this.jobInfo.resourceName, this.sceneid, result);
  if (err) {
    Log.m(TAG, err.stack);
  }
  this.proxy.sendJobExecuteStatus(this.jobInfo.resourceId, result, function (err, status) {
    callback(err);
  });
};

/**
 *  keys = ['tenantId', 'ownerType', 'ownerId', 'bizCode', 'traceId', 'sceneId', 'useTime', 'result', 'timestamp'];
 * @param useTime
 * @param result
 */
Runner.prototype.generateLog = function (useTime, success) {
  var logSummary = util.format("[%s]", (new Date).toLocaleString());
  if (this.config && (this.config)[constant.SCENEOWNER]) {
    try {
      var sceneOwner = (this.config)[constant.SCENEOWNER];
      var tenantId = sceneOwner[constant.CONFIG_TENANTID];
      logSummary += ";tenantId:" + tenantId;
      if (sceneOwner.hasOwnProperty(constant.CONFIG_OWNERTYPE)) {
        var ownerType = sceneOwner[constant.CONFIG_OWNERTYPE];
        logSummary += ";ownerType:" + ownerType;
      }
      if (sceneOwner.hasOwnProperty(constant.CONFIG_OWNERID)) {
        var ownerId = sceneOwner[constant.CONFIG_OWNERID];
        logSummary += ";ownerId:" + ownerId;
      }
    } catch (e) {
      Log.w(TAG, e.stack);
    }
  }
  logSummary += ";bizCode:SUMMARY";
  logSummary += ";traceId:" + this.traceId;
  logSummary += ";sceneId:" + this.sceneid;
  logSummary += ";useTime:" + useTime;
  logSummary += ";result:" + success;
  logSummary += ";timestamp:" + Math.round(new Date().getTime());// 毫秒
  logSummary += "\n";
  return logSummary;
};

Runner.prototype.execute = function (callback) {
  var self = this;
  var time = new Date();
  var contentFile = this.jobInfo.content.file;
  fs.readFile(contentFile, { encoding: 'utf-8' }, function (err, contentStr) {
    if (err) {
      return self.sendJobStatus(err, time, '', callback);
    }

    Log.i(TAG + ' Read content file %s.', contentStr);

    var sceneObj;
    try {
      sceneObj = JSON.parse(contentStr);
    } catch (err) {
      return self.sendJobStatus(err, time, '', callback);
    }
    if (!self.isValid(sceneObj)) {
      return self.sendJobStatus(new Error('Not a valid scene: ' + contentStr), time, '',
        callback);
    }

    var actionsObj = sceneObj[constant.ACTION];
    var action = new Action(actionsObj, self.sceneid, self.proxy, self.traceId);
    action.execute(function (err, detail) {
      Log.i(TAG + ' %s(%s) execute result: err=%s, detail=%s.',
        self.jobInfo.resourceName, self.jobInfo.resourceId, err, detail);
      return self.sendJobStatus(err, time, detail, callback);
    });
  });
};

function run() {
  Log.i(TAG, ' Start to execute job ', argv.info);

  try {
    var jobInfo = JSON.parse(argv.info);
  } catch (e) {
    Log.w(TAG + ' Not JSON string: ', argv.info, e.stack);
    process.exit(-1);
  }

  Log.init(jobInfo.resourceName);

  new Runner(jobInfo, proxy).execute(function (err) {
    if (err) Log.i(TAG, err.stack);
    process.exit(0);
  });
}

run();
