'use strict';
const WebSocket = require('ws');
const constant = require("../../utils/constant");
const broker = require("../../broker");
const Log = require("../../utils/log");
const getUUID = require("../getUUID");
const TAG = "[WSClient]";

class WSClient {
  constructor(serverPath) {
    this.wss = new WebSocket(serverPath);
    this.pendingCalls = {};
    this.queue = [];
    this.opened = false;
    this.wss.on('open', () => {
      this.queue.forEach((msg) => {
        this.wss.send(msg);
      });
      this.queue = [];
      this.opened = true;
    });

    this.wss.on('close', () => {
      Log.i(TAG, 'On close event.');
      this.opened = false;
    });

    this.wss.on('message', (data) => {
      try {
        var obj = JSON.parse(data);
        this.handleIncomingMessage(obj);
      } catch (e) {
        Log.w(TAG, `On message error: `, e.message);
      }
    });

    this.wss.on('error', (err) => {
      Log.w(TAG, 'On error event: ' + err);
    });
  };

  handleIncomingMessage(msg) {
    var tag = msg['tag'];
    // Log.i("WSClient", "receiveMessage", JSON.stringify(msg), "tag", tag);
    var status = msg['status'];
    var cb = this.pendingCalls[tag];
    if (cb) {
      if (status === 'success') {
        cb(undefined, msg['value']);
      } else {
        cb(new Error(msg['error']), undefined);
      }
      delete this.pendingCalls[tag];
    }

    /**
     * 消息路由过来的消息 JsonObject
     * msg : {
         *    functionId : "",
         *    event : {
         *        payload : {},
         *        topic : " "
         *    }
         * }
     * @param msg
     */
    if (msg.hasOwnProperty(constant.CONFIG_FUNCTIONID)) {
      // var newMsg = {topic : msg.topic, payload : msg.payload};
      var functionId = msg[constant.CONFIG_FUNCTIONID];
      var newMsg = JSON.parse(JSON.stringify(msg));
      delete newMsg[constant.CONFIG_FUNCTIONID];
      broker.publish(functionId, newMsg);
      Log.i("WSClient", "broker publish functionid = " + msg[constant.CONFIG_FUNCTIONID]);
    }
  };

  send(msg) {
    // Log.i("WSClient", "this.opened = " + this.opened);
    if (this.opened) {
      this.wss.send(msg);
    } else {
      this.queue.push(msg);
    }
  };

  setProperty(id, values, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;

    var request = {
      tag: tag,
      cmd: "setProperty",
      id: id,
      values: values
    };
    Log.i(TAG, `WSClient.setProperty(${id}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  };

  getProperty(id, keys, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;

    var request = {
      tag: tag,
      cmd: "getProperty",
      id: id,
      keys: keys
    };
    Log.i(TAG, `WSClient.getProperty(${id}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));

  };

  callService(id, service, params, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;

    var request = {
      tag: tag,
      cmd: "callService",
      id: id,
      service: service,
      params: params
    };
    Log.i(TAG, `WSClient.callService(${id}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  };

  sendJobExecuteStatus(id, status, cb) {
      var tag = getUUID();
      this.pendingCalls[tag] = cb;
      var request = {
      tag: tag,
      cmd: "sendJobExecuteStatus",
      id: id,
      status: status
    };
    // console.log(`WSClient.sendJobExecuteStatus(${id}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  };

  getDeviceList(status, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;

    var request = {
      tag: tag,
      cmd: "getDeviceList",
      status: status,
    };
    Log.i(TAG, `WSClient.getDeviceList(${status}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  };

  getDeviceProfile(id, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;

    var request = {
      tag: tag,
      cmd: "getDeviceProfile",
      id: id,
    };
    Log.i(TAG, `WSClient.getDeviceProfile(${id}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  };

  publish(wellknownName, _interface, objectPath, messageName, payload, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;
    var request = {
      tag: tag,
      cmd: "publish",
      wellknownName: wellknownName,
      _interface: _interface,
      objectPath: objectPath,
      messageName: messageName,
      payload: payload
    };
    Log.i(TAG, `WSClient.publish(${messageName}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  }

  notifyIotMessage(message, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;
    var request = {
      tag: tag,
      cmd: "notifyIotMessage",
      message: message
    };
    Log.i(TAG, `WSClient.notifyIotMessage(${message}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  }

  operateScene(id, operation, cb) {
    var tag = getUUID();
    this.pendingCalls[tag] = cb;
    var request = {
      tag: tag,
      cmd: "operateScene",
      id: id,
      operation: operation
    };
    Log.i(TAG, `WSClient.operateScene(${id}, ${operation}, ${JSON.stringify(request)})`);
    this.send(JSON.stringify(request));
  }


  /**
   * The method forwards client's http like requests to WebSocket server.
   *
   * @param options {Object} the http like {@link http.request#options request}.
   * @param callback the callback to receive the result.
   */
  request(options, callback) {
    var tag = getUUID();
    var rcb = function (err, res) {
      Log.i(`Response received for request "${tag}".`);
      if (!res) {
        callback(new Error(`Incorrect response: ${res}.`));
        return;
      }
      if (res.body) {
        res.body = Buffer.from(res.body, 'base64');
      }
      callback(null, res);
    };
    this.pendingCalls[tag] = rcb;
    var request = {
      tag: tag,
      cmd: "request",
      options: options
    };
    if (!options.headers) {
      // Headers should always exist.
      options.headers = {};
    }
    if (options.body) {
      // The body should be base64 encoded since we here send messages as string.
      request.options.body = Buffer.from(options.body).toString('base64');
    }
    var message = JSON.stringify(request);
    Log.i(TAG, `request(${message})`);
    this.send(message);
  }
}

const SrvPath = 'ws://127.0.0.1:8099';
module.exports = new WSClient(SrvPath);
