const Tag = require('./utils/constant');
const Log = require('./utils/log');
const Trigger = require('./trigger.js');
const util = require('util');
var condition = require("./condition");

var Scene = function (scheduler, eventBus, statemachine) {
  this.sceneid = 0; // 场景编号
  this.enabled = true; // 是否被允许
  this.operation = null; // 逻辑符，and/or
  this.triggers = []; // 条件列表
  this.jobInfo = {}; // 与jobmanager沟通的信息
  this.scheduler = scheduler;
  this.eventBus = eventBus;
  this.stateMachine = statemachine;
};

Scene.prototype.parseScene = function (sceneObj) {
  Log.i('Parse scene: ' + JSON.stringify(sceneObj));
  if (!this.jobInfo || !this.jobInfo['resourceId'] ||
    !sceneObj.hasOwnProperty(Tag.ACTION)) {
    return false;
  }

  this.sceneid = this.jobInfo['resourceId'];
  this.sceneObj = sceneObj;

  if (sceneObj.hasOwnProperty(Tag.CONDITION)) {
    var conditionObject = sceneObj[Tag.CONDITION];
    if (conditionObject['items'] !== undefined) {
      this.sceneObj[Tag.CONDITION] = conditionObject['items'];
    } else {
      var items = [];
      items.push(conditionObject);
      this.sceneObj[Tag.CONDITION] = items;
    }
  }

  if (!this.parseTriggers(sceneObj[Tag.TRIGGER])) {
    return false;
  }

  return true;
};

Scene.prototype.parseTriggers = function (triggers) {
  if (!triggers || Object.keys(triggers).length === 0) {
    this.operation = this.operation = Tag.LogicOr;
    return true;
  }
  var self = this;
  this.operation = triggers[Tag.URI];
  var condilist = triggers[Tag.ITEMS] || [];
  // items是空的时候，意味着只有一个条件
  if (condilist.length === 0) {
    var condi = {};
    condi[Tag.URI] = triggers[Tag.URI];
    condi[Tag.PARAMS] = triggers[Tag.PARAMS];
    this.operation = Tag.LogicOr; // 只有一个条件的时候，置为or逻辑，与多个条件时兼容
    condilist.push(condi);
  }
  var result = false;
  condilist.forEach(function (condi) {
    var con = new Trigger(condi, self.sceneid, self.eventBus, self.stateMachine);
    result = con.start();
    if (result) {
      self.triggers.push(con);
    }
  });
  return result;
};

Scene.prototype.propertiesChanged = function (info) {
  if (!this.enabled) {
    return;
  }
  var canExecute = false;
  if (this.operation === Tag.LogicOr) {
    canExecute = this.triggers.some(function (condi) {
      return condi.onTrigger(info);
    });
  } else if (this.operation === Tag.LogicAnd) {
    canExecute = this.triggers.every(function (condi) {
      // And逻辑下，需要检查是否本次被触发/或者曾经触发过
      return condi.onTrigger(info) || condi.isFullFilled();
    });
  } else {
    Log.i('Unsupported operation %s for scene %s.', this.operation, this.sceneid);
    canExecute = false;
  }

  Log.d('PropertiesChanged trigger for scene %s, canExecute=%s', this.sceneid, canExecute);
  var self = this;
  if (canExecute) {
    Log.i('Condition=%s', JSON.stringify((this.sceneObj)[Tag.CONDITION]));
    condition.isConditionsMeeted((this.sceneObj)[Tag.CONDITION], function (err, satisfy) {
      Log.i('Meet conditions: sid=%s, err=%s, meet=%s', self.sceneid, err, satisfy);
      if (!err && satisfy) {
        self.executeJob();
      }
    });
  }
};

Scene.prototype.timerTriggered = function (info) {
  // log.i(TAG, "timerTriggered begin info = " + JSON.stringify(info));
  if (this.sceneid !== info.sceneid || !this.enabled) return undefined;
  var self = this;
  condition.isConditionsMeeted((this.sceneObj)[Tag.CONDITION], function (err, satisfy) {
    Log.i('Meet conditions: sid=%s, err=%s, meet=%s', self.sceneid, err, satisfy);
    if (!err && satisfy) {
      self.executeJob();
    }
  });
};

Scene.prototype.eventHappened = function (info) {
  Log.d('EventHappened trigger for scene %s, enabled=%s, info=%s', this.sceneid,
    this.enabled, JSON.stringify(info));
  if (!this.enabled) {
    return;
  }

  var canExecute = false;
  if (this.operation === Tag.LogicOr) {
    canExecute = this.triggers.some(function (condi) {
      return condi.eventHappened(info);
    });
  } else if (this.operation === Tag.LogicAnd) {
    canExecute = this.triggers.every(function (condi) {
      // And逻辑下，需要检查是否本次被触发/或者曾经触发过
      return condi.eventHappened(info) || condi.isFullFilled();
    });
  } else {
    Log.i('Unsupported operation %s for scene %s.', this.operation, this.sceneid);
    canExecute = false;
  }

  Log.d('EventHappened trigger for scene %s, canExecute=%s', this.sceneid, canExecute);
  if (canExecute) {
    var self = this;
    condition.isConditionsMeeted((this.sceneObj)[Tag.CONDITION], function (err, satisfy) {
      Log.i('Meet conditions: sid=%s, err=%s, meet=%s', self.sceneid, err, satisfy);
      if (!err && satisfy) {
        self.executeJob();
      }
    });
    // this.executeJob();
  }
};

Scene.prototype.executeJob = function () {
  Log.i('Execute actions for %s...', this.sceneid);
  if (this.scheduler) {
    this.scheduler.schedule(this.jobInfo);
  }
};

Scene.prototype.stop = function () {
  var stopped = true;
  this.triggers.forEach(function (condi) {
    stopped = condi.stop() && stopped;
  });
  return stopped;
};

module.exports = Scene;
